#!/usr/bin/python3
"""
## © University of Southampton IT Innovation Centre, 2018
##
## Copyright in this software belongs to University of Southampton
## IT Innovation Centre of Gamma House, Enterprise Road,
## Chilworth Science Park, Southampton, SO16 7NS, UK.
##
## This software may not be used, sold, licensed, transferred, copied
## or reproduced in whole or in part in any manner or form or in or
## on any media by any person other than in accordance with the terms
## of the Licence Agreement supplied with the software, or otherwise
## without the prior written consent of the copyright owners.
##
## This software is distributed WITHOUT ANY WARRANTY, without even the
## implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
## PURPOSE, except where stated in the Licence Agreement supplied with
## the software.
##
##      Created By :            Michael Boniface
##      Created Date :          15-04-2018
##      Updated By :            Nikolay Stanchev
##      Updated Date :          16-04-2018
##      Created for Project :   FLAME
"""


from influxdb import InfluxDBClient


class Simulator(object):
    """
    Simualator used to generate E2E measurements.
    """

    DATABASE = 'E2EMetrics'
    MINUTE = 60000000000  # a minute in nanoseconds
    SIMULATION_LENGTH = 22  # simulation time in minutes

    def __init__(self, host, port):
        """
        Initialises the simulator by creating a db client object and resetting the database.

        :param host: db host url
        :param port: db port number
        """

        self.db_client = InfluxDBClient(host=host, port=port, database=self.DATABASE, timeout=10)

        self.reset_db()

    def reset_db(self):
        """
        Reset the database using the already initialised db client object.
        """

        self.db_client.drop_database(self.DATABASE)
        self.db_client.create_database(self.DATABASE)

    def aggregate_measurements(self, start_time, end_time, sample_period=2):
        """
        Executes a query to aggregate the measurements after the simulation is over.
        :param start_time: start time of simulation (in nanoseconds)
        :param end_time: end time of simulation (in nanoseconds)
        :param sample_period: sample period for grouping (in minutes)
        :return:
        """

        query = 'SELECT mean("delay") as "Dnet", mean("response_time") as "Dresponse_time" INTO "E2EMetrics"."autogen"."e2e" from "E2EMetrics"."autogen"."net", "E2EMetrics"."autogen"."service" WHERE time >= {0} and time <= {1} GROUP BY time({2}m) fill(previous)'.format(start_time, end_time, sample_period)
        self.db_client.query(query=query)

    def run(self):
        """
        Runs the simulation.
        """

        start_time = 1523779200000000  # 15/04/2018, 09:00:00 in nanoseconds
        sim_time = start_time

        mean_delay_seconds_net = 1  # initial mean network delay
        mean_delay_seconds_media = 10  # initial mean media service delay
        sample_period_net = 2  # sample period for reporting network delays (measured in minutes)
        sample_period_media = 5  # sample period for reporting media service delays (measured in minutes)

        for i in range(0, self.SIMULATION_LENGTH):
            # net delay
            if i % sample_period_net == 0:
                point = [{"measurement": "net",
                          "fields": {
                              "delay": mean_delay_seconds_net
                          },
                          "time": sim_time
                          }]
                self.db_client.write_points(point)
                # increase the delay by 1 every sample
                mean_delay_seconds_net += 1

            # service response time
            if i % sample_period_media == 0:
                point = [{"measurement": "service",
                          "fields": {
                              "response_time": mean_delay_seconds_media
                          },
                          "time": sim_time
                          }]
                self.db_client.write_points(point)

                # increase the delay by 20 every sample
                mean_delay_seconds_media += 20

            # increase the time by one minute
            sim_time += self.MINUTE

        end_time = sim_time - self.MINUTE  # decrement one minute to get the last time a measurement have been reported
        self.aggregate_measurements(start_time, end_time)


if __name__ == "__main__":
    Simulator('203.0.113.100', 8086).run()
